package com.example.nto.controller;

import com.example.nto.dto.UsersDTO;
import com.example.nto.entity.Users;
import com.example.nto.exception.RoleisNotAdmin;
import com.example.nto.repository.UsersRepository;
import com.example.nto.service.UsersService;
import lombok.RequiredArgsConstructor;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.Authentication;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Objects;
import java.util.Optional;

@RestController
@RequestMapping("/api/users")
@RequiredArgsConstructor
public class UsersController {

    private final UsersService usersService;
    private final UsersRepository usersRepository;

    @GetMapping
    public List<UsersDTO> getAllUsers() {
        return usersService.getAllUsers();
    }

    @GetMapping("/get")
    public ResponseEntity<UsersDTO> getUserByUsername(@RequestParam String username, Authentication authentication) {
        String name = authentication.getName();
        Optional<Users> user = usersRepository.findByUsername(name);
        if (user.isPresent()) {
            if(Objects.equals(user.get().getAuthority().getAuthority(), "ROLE_ADMIN")) {
                return ResponseEntity.ok(usersService.getUserByUsername(username));
            }
            throw new RoleisNotAdmin("User with username " + user.get().getUsername() + " is not admin");
        }
        throw new RuntimeException("User is not found");
    }


    @GetMapping("/username/{username}")
    public ResponseEntity<String> getByUsername(@PathVariable String username) {
        UsersDTO usersDTO = usersService.getUserByUsername(username);
        return ResponseEntity.ok("User " + usersDTO.getUsername() + " is registered");
    }

    @GetMapping("/login")
    public ResponseEntity<UsersDTO> login(Authentication authentication) {
        return ResponseEntity.ok(usersService.getUserByUsername(authentication.getName()));
    }
    @PatchMapping("/block")
    public ResponseEntity<String> BlockUser(
            @RequestParam() String username, Authentication authentication) {
        String name = authentication.getName();
        Optional<Users> user = usersRepository.findByUsername(name);
        if (user.isPresent()) {
            if (Objects.equals(user.get().getAuthority().getAuthority(), "ROLE_ADMIN")) {
                return ResponseEntity.ok(usersService.blockUser(username));
            }
            throw new RoleisNotAdmin("User with username " + user.get().getUsername() + " is not admin");

        }
        throw new RuntimeException("User not found");
    }

    @PatchMapping("/unblock")
    public ResponseEntity<String> UnblockUser(
            @RequestParam() String username, Authentication authentication) {
        String name = authentication.getName();
        Optional<Users> user = usersRepository.findByUsername(name);
        if (user.isPresent()) {
            if (Objects.equals(user.get().getAuthority().getAuthority(), "ROLE_ADMIN")) {
                return ResponseEntity.ok(usersService.unblockUser(username));
            }
            throw new RoleisNotAdmin("User with username " + user.get().getUsername() + " is not admin");

        }
        throw new RuntimeException("User not found");
    }



}
