package com.example.nto.service.impl;

import com.example.nto.entity.Code;
import com.example.nto.entity.Pass;
import com.example.nto.entity.Users;
import com.example.nto.exception.RoleisNotAdmin;
import com.example.nto.repository.CodeRepository;
import com.example.nto.repository.PassRepository;
import com.example.nto.repository.UsersRepository;
import com.example.nto.service.CodeService;
import com.example.nto.service.UsersService;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;
import org.springframework.web.server.ResponseStatusException;

import java.time.LocalDateTime;
import java.util.Optional;

@Service
@RequiredArgsConstructor
public class CodeServiceImpl implements CodeService {

    private final CodeRepository codeRepository;
    private final UsersRepository usersRepository;
    private final UsersService usersService;
    private final PassRepository passRepository;

    @Override
    public Boolean findExistByValue(Long value) {
        return codeRepository.findByValue(value).isPresent();
    }

    @Override
    public Optional<Users> openDoor(String username, Long value) {
        if (usersRepository.findByUsername(username).isPresent() && findExistByValue(value)) {
            Optional<Users> user = usersRepository.findByUsername(username);
            Optional<Code> code = codeRepository.findByValue(value);
            if (!user.get().getIsCardBlocked()) {
                Pass pass = new Pass();
                pass.setTerminal(code.get().getTerminal());
                pass.setUsers(user.get());
                pass.setTime(LocalDateTime.now());
                passRepository.save(pass);
                user.get().setLastVisit(LocalDateTime.now());

                return Optional.ofNullable(usersService.updateEmployee(user.get().getId(), user.orElse(null)));
            }
            throw new RoleisNotAdmin("Card is blocked");
        }

        throw new ResponseStatusException(HttpStatus.BAD_REQUEST);

    }
}
